Package: domir
Title: Dominance Analysis Methods
Version: 1.3.0
Date: 2026-4-19
Authors@R: 
    person(given = "Joseph",
           family = "Luchman",
           role = c("aut", "cre"),
           email = "jluchman@gmail.com",
           comment = c(ORCID = "0000-0002-8886-9717"))
Description: Dominance analysis relative importance methods that are intended 
    for predictive models.
Imports: parallel, stats, utils
Suggests: dplyr, dominanceanalysis, forcats, Formula, ggplot2, knitr,
        lme4, parameters, performance, pscl, purrr, relaimpo, rlang,
        rmarkdown, stringr, systemfit, testthat (>= 3.0.0), tidyr
License: GPL (>= 3)
URL: https://github.com/jluchman/domir,
        https://jluchman.github.io/domir/
BugReports: https://github.com/jluchman/domir/issues
Encoding: UTF-8
RoxygenNote: 7.3.3
Config/testthat/edition: 3
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-04-19 15:08:02 UTC; josephluchman
Author: Joseph Luchman [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8886-9717>)
Maintainer: Joseph Luchman <jluchman@gmail.com>
Repository: CRAN
Date/Publication: 2026-04-19 15:20:02 UTC
Built: R 4.6.0; ; 2026-04-25 18:21:34 UTC; unix
