# Generated by roxygen2: do not edit by hand

S3method(domir,formula)
S3method(domir,formula_list)
S3method(print,domin)
S3method(print,domir)
S3method(summary,domin)
S3method(summary,domir)
export(domin)
export(domir)
export(fmllst2Fml)
export(formula_list)
